{#
	SPDX-License-Identifier: MIT

	We copy in the original brand template from Furo here so we can muck with the name
#}
<a class="sidebar-brand{% if logo %} centered{% endif %}" href="{{ pathto(master_doc) }}">
	{%- block brand_content %}
	{#- Remember to update the prefetch logic in `block logo_prefetch_links` in base.html #}
	{%- if logo_url %}
	<div class="sidebar-logo-container">
	  <img class="sidebar-logo" src="{{ logo_url }}" alt="Logo"/>
	</div>
	{%- endif %}
	{%- if theme_light_logo and theme_dark_logo %}
	<div class="sidebar-logo-container">
	  <img class="sidebar-logo only-light" src="{{ pathto('_static/' + theme_light_logo, 1) }}" alt="Light Logo"/>
	  <img class="sidebar-logo only-dark" src="{{ pathto('_static/' + theme_dark_logo, 1) }}" alt="Dark Logo"/>
	</div>
	{%- endif %}
	{%- if current_version %}
	<span class="sidebar-brand-text">{{ project }} {{ current_version.name if current_version.is_released else release }} Documentation</span>
	{%- else -%}
	{% if not theme_sidebar_hide_name %}
	<span class="sidebar-brand-text">{{ docstitle if docstitle else project }}</span>
	{%- endif %}
	{%- endif %}
	{% endblock brand_content %}
  </a>
